function [nEvent power] = sampleSizeCalculation(alpha, x, cs, b, beta, censored)
%
% this function calculate the sample size required in order to ensure a
% sufficient power for the conditional independence tests Ind(x, t | covs)
% For the details, see:
%
% Hsieh, F.Y. and Lavori, P.W. 2000. "Sample size calculation for the cox 
% proportional hazards regression model with nonbinary covariates",
% Controlled Clinical Trials, Volume 21, pages 552-560.
%
% Input:
% alpha: the desired level of significance
% x: the variable subject to the test
% cs: the conditioning set
% b: the coefficient of x given by a cox regression on time t over x and
% cs; in formulas:  b = coeffs(1) where coeffs = coxphfit([x cs],t);
% beta: the required level of power
%
%Output:
% the number of events required to ensure a power of beta.
%

%let's check if the variables is binary
binary = sum(x == 1) + sum(x == 0) == length(x);

%values required by the formula
z_alpha = norminv(alpha);
z_beta = norminv(1-beta);
if binary
    p = sum(x == 1);
    x_var = p*(p-1);
else
    x_var = var(x);
end

%numerator and denominator of the formula
numerator = (z_alpha + z_beta)^2;
denominator = x_var * (b^2);

%number of envent
nEvent = numerator/denominator;

%Variance Inflation Factor (VIF), necessary if the conditioning set is
%present
R2 = 0;
if ~isempty(cs) && ~binary
    [b,bint,r,rint,stats] = regress(x, [cs ones(size(x))]);
    R2 = stats(1);
    nEvent = nEvent/(1-R2);
end

%power: let's calculate the true power
nActualEvent = sum(~censored);
z_beta = sqrt((1-R2)*denominator*nActualEvent) - abs(z_alpha);
power = normcdf(z_beta);