function [Uni,UniD,Gamma,GammaD,logProb,logProbD,PostGamD,MargGam,Wmat,Lmat,accprob]=...
    bvsme_aft(type,X,data,alpha0,beta0,h0,Hg,s20,w,initial,iter,burnin)

%bvsme_surv: Bayesian Variable Selection for AFT models with right censoring
%            Metropolis search, selection prior with diagonal cov matrix 
%			     Main Program 
%***********************************************************************
%INPUTS:
% type          scalar. 1=lognormal data. 2=log-t data (log-logit also possible, see Notes)
% X             predictor matrix (n-by-p) 
% data          data(:,1)=failure times, data(:,2)=censoring indicators
%                                        1 - if data(i,1) is actual value
%                                        0 - if data(i,1) is (right) censoring 
%                                        time (actual value only known to be larger)
%                       
% alphao,beta0  hyperparameters
% h0            scalar (large for vague specification)
% Hg        	hyperparameter (vector) of variance prior on regression coefficients
% s20           prior on sigma2 (v0=3; v2=v0*s20; mean(sigma2)=s20)
% w 		    hyperparameter of prior on latent binary vector
%		        - scalar in the case of Bernoulli priors with equal parameters w_j=w 
%		        (Binomial prior with mean pw)
%		        - vector of two values for a Beta-Binomial prior
% 		        (with mean=pw(1), w(2)=sum of the two parameters of the Beta)
% initial    initial number of included variables
% iter       number of MCMC iterations
% burnin     burn-in
%
%OUTPUTS:
% Uni		vector with number of ones of visited models
% UniD		vector with number of ones of distinct visited models
% Gamma         all visited models
% GammaD	distinct visited models, ordered according to
%		    their (normalized) relative posterior probability)
% logProb       log-relative posterior probabilities 
%		    of all visited models
% logProbD      log-relative posterior probabilities
%                   of distinct visited models
% PostGamD	normalized ordered relative probabilities 
%		    of distinct visited models 
% MargGam       marginal posterior probabilities of inclusion of single vars
% Wmat,Lmat
% accprob       proportion of accepted moves
%
%NOTES: 
% For small 'df' may get warnings from betainv due to the Netwon procedure failing to converge.
% for log-t (type=2) nu is set to 8 (line 76 below, used only in genlam)
% the code can be used for log-logit by changing to d=1.57 in line 74 below (for nu=8)
% Gamma and GammaD are in cell array format 
%  (elements are strings --> required by unique.m)
% Search for replicates among visited accepted models of same size
% Functions called: logpostgam, metro_surv, genlam, multrunct, replichef, postinfer
%
%
%REFERENCE:
%
% Sha, N., Tadesse, M.G. and Vannucci, M. (2006). Bayesian variable selection for the 
% analysis of microarray data with consored outcome. 
% Bioinformatics, 22(18), 2262-2268.
%     
%Copyright (c) 2006 Naijun Sha and Marina Vannucci.
%**********************************************************************

[n,p]=size(X);
T=data(:,1);     %---failure times
dcen=data(:,2);  %---censoring indicators
Y=log(T);        %---log transform for linearity (log-normal, log-logistic)
W=Y;             %---Initial latent log failure time

%-----IG prior on sigma2
v0=3;
v2=v0*s20;

d=1; %change d to 1.57 for log-logit and use nu=8;
if type==2 %---initial value for lamb for log-t case---
   nu=8; 
   lambda=1./gamrnd(nu/2,2/nu,1,n);
   Lmat=zeros(iter+1,n);
   Lmat(1,:)=lambda;
else
lambda=[];
Lmat=[];
end

%---starting value of variable index r

r=startr(initial,p);  
Gamma=cell(iter+1,1);
logProb=zeros(1,iter+1);
Uni=zeros(1,iter+1);
Wmat=zeros(iter+1,n);

%---log posterior of model r----

Xr=X(:,r);beta0r=beta0(r);
[logprob]=logpostgam(type,W,Xr,p,r,alpha0,h0,beta0r,Hg,lambda,v0,v2,w,d);

Gamma{1}=num2str(r);
logProb(1)=logprob;
Wmat(1,:)=W';
Uni(1)=length(r);
accnum=0;

%------MCMC algorithm-------

disp(' ')
disp('------- RUNNING METROPOLIS ...')
disp(' ')

for i=1:iter
    if i/500==round(i/500)
       disp(' ')
       disp('Number of iterations:')
       disp(i)
       disp('...')
    else
    end

%----update variable selection index r and log posterior value of r------

    [logprob,r,accnum]=metro_aft(type,r,W,Y,X,p,h0,Hg,lambda,v0,v2,w,alpha0,beta0,accnum,d);
    logProb(i+1)=logprob;
    Gamma{i+1}=num2str(r);
    Uni(i+1)=length(r);

%----update latent variable W------

    Xr=X(:,r);
    beta0r=beta0(r);
    pr=length(r);
    Hgr=Hg(r,r);
      switch type %---distribution type----
        case 1        %---log-normal data---
          Pr=eye(n)+h0*ones(n,n)+Xr*Hgr*Xr';
        case 2        %---log student-t with df=nu--
          lambda=genlam(nu,d,W,X,r,alpha0,h0,beta0,Hg,lambda,v0,v2);
          Lmat(i+1,:)=lambda;
          L=diag(lambda);
          Pr=d^2*L+h0*ones(n,n)+Xr*Hgr*Xr';
      end
    mu=alpha0*ones(n,1)+Xr*beta0r;
    W=multrunct(W,mu,v0,Pr,v2,dcen,Y);
    Wmat(i+1,:)=W';
end

%----End of MCMC------

accprob=accnum/iter;
disp(' ')
disp('------- Searching for replicates ...')
disp(' ')

[UniD,GammaD,logProbD]=replichef(Uni(burnin+1:end),Gamma(burnin+1:end), logProb(burnin+1:end));

%----recompute posterior f(r|X,W)----
What=(mean(Wmat(burnin+1:end,:)))'; 
if type==2
    lhat=mean(Lmat(burnin+1:end,:));
else
    lhat=[];
end

[MargGam,UniD,GammaD,PostGamD]=postinfer(type,X,data,UniD,GammaD,What,lhat,alpha0,beta0,h0,Hg,s20,w,d);

