function ibs = IBS(model, prediction, data, time, censored)
	% Function for calculating the integrated Brier Score; see
	% E. Graf et al., "ASSESSMENT AND COMPARISON OF PROGNOSTIC
	% CLASSIFICATION SCHEMES FOR SURVIVAL DATA", Statist. Med. 18, 2529-2545
	% (1999)
	% L. Kronek, A. Reddy, "Logical analysis of survival data: prognostic 
	% survival models by detecting high-degree interactions in right-censored 
	% data", Bioinformativs 24, pages i248i253
	%
	% Input:
	% model: it must contain S, a matrix with th same number of row of data;
	% each row is the Survival function evaluated at the respective
	% timeOfInterest
	% It must contain also timeOfInterest, i.e. time point where an event
	% occurs in the training data
	% data: the covariates of test instances
	% time: the survival time of test instances
	% censored: a binary vector with 1 if the test instances is censored, 0 
	% otherwise

	%
	% Output:
	% IBS: the integrated IBS score.
	%
	% Operation: 
	% the IBS accepts the survival function S of the test instances 
	% and then compares S with the their actual survival times.
	% At the end, all the differences are integrated.

	%let's transpose the data...
	time = time';
	censored = censored';
	btime = sort(time);

	%retrieving the S matrices and the timeOfInterest
	S = model.S';
	timeOfInterest = model.timeOfInterest';

	%let's transform S. It must follow btime
	S = transform_S(S, timeOfInterest, btime);
	
	%let's sort the data (IBS requires only S, time and censored)
	[time ot] = sort(time); %#ok<UDIM>
	cens = ~censored(ot); %cens = 1 -> event; cens = 0 -> censored
	survs= S(:,ot);
    
	%let's find the Survival function for the censoring process
	[csurv_temp xtimes] = ecdf(time, 'censoring', cens, 'function', 'survivor');
	csurv_temp = csurv_temp(2:end)'; %matlabe returns a fake first value
	xtimes = xtimes(2:end)';
	
    %let's find the csurv for all the the element of btime
	csurv = -1*ones(1, length(time));
	for i = 1:length(time)
		time_point = time(i);
		if time_point < min(xtimes)
		    csurv(i) = 1;
		else
		    indices = find(time_point >= xtimes);
		    index = indices(end);
		    csurv(i) = csurv_temp(index);
		end
	end
	csurv(csurv == 0) = Inf;
   
    %finding the single brier scores
	bsc = zeros(1,length(btime));
    for i = 1:length(btime)
		
        help1 = (time <= btime(i) & cens == 1);
		help2 = time > btime(i);		
		bsc(i) = mean((0 - survs(i,:)).^2 .* help1 .* (1 ./ csurv) + (1 - survs(i,:)).^2 .* help2 .* (1 ./ csurv(i)));
    
    end
    
    %let's integrate!
	diffs = [btime(1), btime(2:length(btime)) - btime(1:(length(btime) - 1))];
	ibs = sum(diffs .* bsc)/max(btime);
	
end

%subfunction for transforming S
function S_new = transform_S(S, timeOfInterest, btime)
	
	num_time_points = length(btime);
	num_subjects = size(S,2);

	%let's initialize S_new : num_time_points * num_subjects
	S_new = -1*ones(num_time_points, num_subjects);
	
	%let's calculate the new S!
	for i = 1:num_time_points

        idx = find(btime(i) >= timeOfInterest,1,'last');
		if isempty(idx)
			S_new(i,:) = ones(num_subjects,1);
		else
			S_new(i,:) = S(idx,:);
		end
		
	end

end
